#include "Camera.h"

#include "Body.h"
#include "Physics.h"


/**
 * The getViewMatrix function returns the view matrix based on the camera mode and parameters.
 * 
 * @param whiteball The `whiteball` parameter is a `std::shared_ptr` to a `Body` object. It is used as
 * a reference to the white ball in the scene.
 * @param theta The theta parameter represents the horizontal angle of rotation around the y-axis. It
 * determines the camera's position in the x-z plane.
 * @param phi The parameter `phi` represents the azimuthal angle, which determines the horizontal
 * rotation of the camera around the target object. It is typically measured in radians and can range
 * from 0 to 2π.
 * 
 * @return a glm::mat4 object, which represents the view matrix.
 */
glm::mat4 Camera::getViewMatrix( std::shared_ptr<Body> whiteball, float theta, float phi )
{
    if (m_cameraMode == CameraMode::BirdsEye)
        return birdsEyeView();
    else if (m_cameraMode == CameraMode::Shooting)
        return shootingView( whiteball, theta, phi );
    else // if (m_cameraMode == CameraMode::Trackball)
        return freeLookView( theta, phi );
}

/**
 * The function `birdsEyeView()` returns a transformation matrix that positions the camera in a
 * birds-eye view above the billiard table.
 * 
 * @return a glm::mat4 object, which represents a 4x4 transformation matrix that transforms points
 * from world-space to camera-space.
 */
glm::mat4 Camera::birdsEyeView()
{
    /**
     * TODO: Kamera: Vogelperspektive
     * 
     * Implementieren Sie die Kamerasteuerung in dem die Kamera statisch
     * überhalb des Tisches mit Abstand BirdsEyeDistance gerade nach unten
     * zeigen lassen.
     * glm::lookAt() darf hier nicht verwendet werden, nutzen Sie stattdessen
     * die elementaren Transformationen.
     */

    static constexpr float BirdsEyeDistance = 13.f;
    return glm::identity<glm::mat4>(); // Soll von Welt- in Kamera-space transformieren
}

/**
 * The function `freeLookView` implements a simplified trackball camera control by converting polar
 * coordinates to Cartesian coordinates and returning a view matrix.
 * 
 * @param phi The parameter `phi` represents the azimuthal angle, which determines the horizontal
 * rotation of the camera around the origin. It ranges from 0 to 360 degrees.
 * @param theta The theta parameter represents the azimuthal angle, which determines the horizontal
 * rotation of the camera around the origin. It ranges from 0 to 360 degrees.
 * 
 * @return a glm::mat4 object, which represents a 4x4 transformation matrix that transforms points
 * from world-space to camera-space.
 */
glm::mat4 Camera::freeLookView( float phi, float theta )
{
    /**
     * TODO: Kamera: Vereinfachter Trackball
     * 
     * Implementieren Sie die Kamerasteuerung in dem die Kamera auf einer
     * gedachten Halbkugel oberhalb des Billardtisches entlang bewegt wird.
     * Die Winkel phi und theta werden durch die Anwendung bereits
     * mit den Winkeln in Grad gefüllt. Diese müssen als Polarkoordinaten
     * interpretiert und in kartesische Koordinaten mit Radius m_radius um-
     * gewandelt werden. Der Mittelpunkt soll bei (0,0,0) in Weltkoordinaten
     * liegen.
     * Die Funktion glm::lookAt(eye,center,up) ist hilfreich und soll und
     * kann genutzt werden.
     */

    return glm::identity<glm::mat4>(); // Soll von Welt- in Kamera-space transformieren
}

/**
 * The shootingView function in the Camera class implements a camera control that moves the camera
 * along a hemisphere above the billiard table based on the given phi and theta angles.
 * 
 * @param whiteball A shared pointer to a "Body" object representing the white ball in a billiards
 * game.
 * @param phi The parameter `phi` represents the azimuthal angle, which determines the horizontal
 * position of the camera around the white ball. It ranges from 0 to 360 degrees.
 * @param theta The theta parameter represents the azimuthal angle, which determines the horizontal
 * position of the camera around the white ball. It ranges from 0 to 360 degrees.
 * 
 * @return a glm::mat4 object, which represents a 4x4 transformation matrix that transforms points
 * from world-space to camera-space.
 */
glm::mat4 Camera::shootingView( std::shared_ptr<Body> whiteball, float phi, float theta )
{
    /**
     * TODO: Kamera: Schusskamera
     * 
     * Implementieren Sie die Kamerasteuerung in dem die Kamera auf einer
     * gedachten Halbkugel oberhalb des Billardtisches entlang bewegt wird.
     * Die Winkel phi und theta werden durch die Anwendung bereits
     * mit den Winkeln in Grad gefüllt. Diese müssen als Polarkoordinaten
     * interpretiert und in kartesische Koordinaten mit Radius RadiusShooting
     * umgewandelt werden.
     * Anders als bei freeLookView soll hier der Mittelpunkt der Halbkugel
     * die Position der weißen Kugel sein.
     * Die Funktion glm::lookAt(eye,center,up) ist hilfreich und soll und
     * kann genutzt werden.
     */

    static constexpr float RadiusShooting = 3.0f;
    return glm::identity<glm::mat4>(); // Soll von Welt- in Kamera-space transformieren
}

